from distutils.core import setup

setup(
	name = 'pyquering',
	packages = ['pyquering'], # this must be the same as the name above
	version = '0.1.4',
	description = 'A lib to easy quering and parse it to SQL language or JSON or whatever it is necessary',
	author = 'Thiago Moraes Stefani',
	author_email = 'thiagoms.br@gmail.com',
	url = 'https://github.com/python-libraries/pyquering', # use the URL to the github repo
	download_url = '', # I'll explain this in a second
	keywords = ['quering', 'sql', 'json', 'sql to json', 'json to sql', 'json2sql', 'sql2json'], # arbitrary keywords
	classifiers = [],
)